from macaroonbakery import httpbakery

from theblues.utils import (
    ensure_trailing_slash,
    make_request,
    DEFAULT_TIMEOUT,
)


class JIMM(object):

    def __init__(self, url, timeout=DEFAULT_TIMEOUT, client=None,
                 cookies=None):
        """Initializer.

        @param url The url to the JIMM API.
        @param timeout How long to wait before timing out a request in seconds;
            a value of None means no timeout.
        @param client (httpbakery.Client) holds a context for making http
        requests with macaroons.
        @param cookies (which act as dict) holds cookies to be sent with the
        requests.
        """
        self.url = ensure_trailing_slash(url)
        self.timeout = timeout
        self.cookies = cookies
        if client is None:
            client = httpbakery.Client()
        self._client = client

    def list_models(self, macaroons):
        """ Get the logged in user's models from the JIMM controller.

        @param macaroons The discharged JIMM macaroons.
        @return The json decoded list of environments.
        """
        return make_request("{}model".format(self.url), timeout=self.timeout,
                            client=self._client, cookies=self.cookies)



import base64
import json
import logging
try:
    from urllib import quote
except ImportError:
    from urllib.parse import quote

from theblues.errors import (
    InvalidMacaroon,
    ServerError,
)
from theblues.utils import (
    ensure_trailing_slash,
    make_request,
    DEFAULT_TIMEOUT,
)


class IdentityManager(object):
    """Identity Manager API."""

    def __init__(self, url, timeout=DEFAULT_TIMEOUT):
        """Initializer.

        @param url The url to the identity manager (IdM) API.
        @param timeout How long to wait before timing out a request in seconds;
            a value of None means no timeout.
        """
        self.url = ensure_trailing_slash(url)
        self.timeout = timeout

    def get_user(self, username, macaroons):
        """Fetch user data.

        Raise a ServerError if an error occurs in the request process.

        @param username the user's name.
        @param macaroons the encoded macaroons string.
        """
        url = '{}u/{}'.format(self.url, username)
        return make_request(url, timeout=self.timeout, macaroons=macaroons)

    def debug(self):
        """Retrieve the debug information from the identity manager."""
        url = '{}debug/status'.format(self.url)
        try:
            return make_request(url, timeout=self.timeout)
        except ServerError as err:
            return {"error": str(err)}

    def login(self, username, json_document):
        """Send user identity information to the identity manager.

        Raise a ServerError if an error occurs in the request process.

        @param username The logged in user.
        @param json_document The JSON payload for login.
        """
        url = '{}u/{}'.format(self.url, username)
        make_request(
            url, method='PUT', body=json_document, timeout=self.timeout)

    def discharge(self, username, macaroon):
        """Discharge the macarooon for the identity.

        Raise a ServerError if an error occurs in the request process.

        @param username The logged in user.
        @param macaroon The macaroon returned from the charm store.
        @return The resulting base64 encoded macaroon.
        @raises ServerError when making request to the discharge endpoint
        InvalidMacaroon when the macaroon passedin or discharged is invalid
        """
        caveats = macaroon.third_party_caveats()
        if len(caveats) != 1:
            raise InvalidMacaroon(
                'Invalid number of third party caveats (1 != {})'
                ''.format(len(caveats)))
        url = '{}discharger/discharge?discharge-for-user={}&id={}'.format(
            self.url, quote(username), caveats[0][1])
        logging.debug('Sending identity info to {}'.format(url))
        logging.debug('data is {}'.format(caveats[0][1]))
        response = make_request(url, method='POST', timeout=self.timeout)
        try:
            macaroon = response['Macaroon']
            json_macaroon = json.dumps(macaroon)
        except (KeyError, UnicodeDecodeError) as err:
            raise InvalidMacaroon(
                'Invalid macaroon from discharger: {}'.format(err.message))

        return base64.urlsafe_b64encode(json_macaroon.encode('utf-8'))

    def discharge_token(self, username):
        """Discharge token for a user.

        Raise a ServerError if an error occurs in the request process.

        @param username The logged in user.
        @return The resulting base64 encoded discharged token.
        """
        url = '{}discharge-token-for-user?username={}'.format(
            self.url, quote(username))
        logging.debug('Sending identity info to {}'.format(url))
        response = make_request(url, method='GET', timeout=self.timeout)
        try:
            macaroon = response['DischargeToken']
            json_macaroon = json.dumps(macaroon)
        except (KeyError, UnicodeDecodeError) as err:
            raise InvalidMacaroon(
                'Invalid macaroon from discharger: {}'.format(err.message))
        return base64.urlsafe_b64encode("[{}]".format(
            json_macaroon).encode('utf-8'))

    def _get_extra_info_url(self, username):
        """Return the base URL for extra-info requests.

        @username The user who's information is being accessed.
        """
        return '{}u/{}/extra-info'.format(self.url, username)

    def set_extra_info(self, username, extra_info):
        """Set extra info for the given user.

        Raise a ServerError if an error occurs in the request process.

        @param username The username for the user to update.
        @param info The extra info as a JSON encoded string, or as a Python
            dictionary like object.
        """
        url = self._get_extra_info_url(username)
        make_request(url, method='PUT', body=extra_info, timeout=self.timeout)

    def get_extra_info(self, username):
        """Get extra info for the given user.

        Raise a ServerError if an error occurs in the request process.

        @param username The username for the user who's info is being accessed.
        """
        url = self._get_extra_info_url(username)
        return make_request(url, timeout=self.timeout)



import collections
import json
try:
    from urllib import urlencode
except ImportError:
    from urllib.parse import urlencode

import requests
from requests.exceptions import HTTPError

from theblues.errors import (
    log,
    ServerError,
    timeout_error,
)


API_URL = 'https://api.jujucharms.com/charmstore/v5'
DEFAULT_TIMEOUT = 3.05
_error_message = 'Error during request: {url} message: {message}'


def _server_error_message(url, message):
    """Log and return a server error message."""
    msg = _error_message.format(url=url, message=message)
    log.error(msg)
    return msg


def make_request(
        url, method='GET', query=None, body=None, auth=None, timeout=10,
        client=None, macaroons=None):
    """Make a request with the provided data.

    @param url The url to make the request to.
    @param method The HTTP request method (defaulting to "GET").
    @param query A dict of the query key and values.
    @param body The optional body as a string or as a JSON decoded dict.
    @param auth The optional username and password as a tuple,
    not used if client is not None
    @param timeout The request timeout in seconds, defaulting to 10 seconds.
    @param client (httpbakery.Client) holds a context for making http
    requests with macaroons.
    @param macaroons Optional JSON serialized, base64 encoded macaroons to be
        included in the request header.

    POST/PUT request bodies are assumed to be in JSON format.
    Return the response content as a JSON decoded object, or an empty dict.
    Raise a ServerError if a problem occurs in the request/response process.
    Raise a ValueError if invalid parameters are provided.
    """
    headers = {}
    kwargs = {'timeout': timeout, 'headers': headers}
    # Handle the request body.
    if body is not None:
        if isinstance(body, collections.Mapping):
            body = json.dumps(body)
        kwargs['data'] = body
    # Handle request methods.
    if method in ('GET', 'HEAD'):
        if query:
            url = '{}?{}'.format(url, urlencode(query, True))
    elif method in ('DELETE', 'PATCH', 'POST', 'PUT'):
        headers['Content-Type'] = 'application/json'
    else:
        raise ValueError('invalid method {}'.format(method))
    if macaroons is not None:
        headers['Macaroons'] = macaroons

    kwargs['auth'] = auth if client is None else client.auth()

    api_method = getattr(requests, method.lower())
    # Perform the request.
    try:
        response = api_method(url, **kwargs)
    except requests.exceptions.Timeout:
        raise timeout_error(url, timeout)
    except Exception as err:
        msg = _server_error_message(url, err)
        raise ServerError(msg)
    # Handle error responses.
    try:
        response.raise_for_status()
    except HTTPError as err:
        msg = _server_error_message(url, err.response.text)
        raise ServerError(err.response.status_code, msg)
    except requests.exceptions.RequestException as err:
        msg = _server_error_message(url, err.message)
        raise ServerError(msg)
    # Some requests just result in a status with no response body.
    if not response.content:
        return {}
    # Assume the response body is a JSON encoded string.
    try:
        return response.json()
    except Exception as err:
        msg = 'Error decoding JSON response: {} message: {}'.format(url, err)
        log.error(msg)
        raise ServerError(msg)


def ensure_trailing_slash(url):
    """Returns a url with a trailing slash

    @param url The url to ensure has a trailing slash
    """
    if not(url.endswith('/')):
        url += '/'
    return url



from email.utils import parseaddr
import requests
from requests.exceptions import (
    RequestException,
    Timeout,
    )

from theblues.errors import (
    log,
    ServerError,
)
from theblues.utils import (
    ensure_trailing_slash,
    DEFAULT_TIMEOUT,
)


class Priority:
    L1 = "L1- Core functionality not available"
    L2 = "L2- Core functionality severely degraded"
    L3 = "L3- Standard support request"
    L4 = "L4- Non-urgent issue"
    L5 = "L5"
    Undecided = "Undecided"
    Wishlist = "Wishlist"
    Low = "Low"
    Medium = "Medium"
    High = "High"
    Critical = "Critical"


class Support(object):

    # This represent the field name for business impact in SalesForce.
    BUSINESS_IMPACT = '00ND0000005lqBV'

    def __init__(self, url, orgId, recordType, timeout=DEFAULT_TIMEOUT):
        """Initializer.

        @param url The url to the Support server.
        @param orgId the organization Id for SalesForce.
        @param recordType the record type.
        @param timeout How long to wait before timing out a request in seconds;
            a value of None means no timeout.
        """
        self.url = ensure_trailing_slash(url)
        self.orgId = orgId
        self.recordType = recordType
        self.timeout = timeout

    def create_case(self, name, email, subject, description, businessImpact,
                    priority, phone):
        """ Send a case creation to SalesForces to create a ticket.

        @param name of the person creating the case.
        @param email of the person creating the case.
        @param subject of the case.
        @param description of the case.
        @param businessImpact of the case.
        @param priority of the case.
        @param phone of the person creating the case.
        @return Nothing if this is ok.
        @raise ServerError when something goes wrong.
        @raise ValueError when data passed in are invalid
        """

        if not('@' in parseaddr(email)[1]):
            raise ValueError('invalid email: {}'.format(email))
        if '' == name or name is None:
            raise ValueError('empty name')
        if '' == subject or subject is None:
            raise ValueError('empty subject')
        if '' == description or description is None:
            raise ValueError('empty description')
        if '' == businessImpact or businessImpact is None:
            raise ValueError('empty business impact')
        if priority is None:
            raise ValueError('Ensure the priority is from the set of '
                             'known priorities')
        if '' == phone or phone is None:
            raise ValueError('empty phone')

        try:
            r = requests.post(self.url, data={
                'orgid': self.orgId,
                'recordType': self.recordType,
                'name': name,
                'email': email,
                'subject': subject,
                'description': description,
                self.BUSINESS_IMPACT: businessImpact,
                'priority': priority,
                'phone': phone,
                'external': 1
                }, timeout=self.timeout)
            r.raise_for_status()
        except Timeout:
            message = 'Request timed out: {url} timeout: {timeout}'
            message = message.format(url=self.url, timeout=self.timeout)
            log.error(message)
            raise ServerError(message)
        except RequestException as err:
            log.info('cannot create case: {}'.format(err))
            raise ServerError(
                'cannot create case: {}'.format(err))



#!/usr/bin/env python

import os
import sys

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

readme = open('README.rst').read()
history = open('CHANGELOG.rst').read().replace('.. :changelog:', '')

setup(
    name='theblues',
    version='0.5.2',
    description='Python library for using the juju charm store API.',
    long_description=readme + '\n\n' + history,
    author='Juju GUI Developers',
    author_email='juju-gui@lists.ubuntu.com',
    url='https://github.com/juju/theblues',
    packages=[
        'theblues',
    ],
    package_dir={'theblues': 'theblues'},
    include_package_data=True,
    install_requires=[
        'requests>=2.18.4',
        'jujubundlelib>=0.5.1',
        'macaroonbakery>=0.0.6',
    ],
    tests_requires=[
        'httmock==1.2.3',
    ],
    zip_safe=False,
    keywords='theblues',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Lesser General Public License v3 ' +
        '(LGPLv3)',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
    ],
)
